@@ -1,5 +1,6 @@
 /*
  * synergy -- mouse and keyboard sharing utility
+ * Copyright (C) 2012 Bolton Software Ltd.
  * Copyright (C) 2008 Volker Lanz (vl@fidra.de)
  * 
  * This package is free software; you can redistribute it and/or
@@ -15,20 +16,30 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#define WEBSITE_ADDRESS "synergy-foss.org"
+
+#include <iostream>
+
 #include "MainWindow.h"
 #include "AboutDialog.h"
 #include "ServerConfigDialog.h"
 #include "SettingsDialog.h"
-#include "WindowsServices.h"
+#include "SetupWizard.h"
 
 #include <QtCore>
 #include <QtGui>
 #include <QtNetwork>
+#include <QNetworkAccessManager>
 
 #if defined(Q_OS_MAC)
 #include <ApplicationServices/ApplicationServices.h>
 #endif
 
+#if defined(Q_OS_WIN)
+#define WIN32_LEAN_AND_MEAN
+#include <Windows.h>
+#endif
+
 #if defined(Q_OS_WIN)
 static const char synergyConfigName[] = "synergy.sgc";
 static const QString synergyConfigFilter(QObject::tr("Synergy Configurations (*.sgc);;All files (*.*)"));
@@ -44,64 +55,110 @@ static const char* synergyIconFiles[] =
 	":/res/icons/16x16/synergy-connected.png"
 };
 
-class QThreadImpl : public QThread
-{
-public:
-	static void msleep(unsigned long msecs)
-	{
-		QThread::msleep(msecs);
-	}
-};
-
-MainWindow::MainWindow() :
-	m_Settings(),
-	m_AppConfig(&m_Settings),
+MainWindow::MainWindow(QSettings& settings, AppConfig& appConfig) :
+	m_Settings(settings),
+	m_AppConfig(appConfig),
 	m_pSynergy(NULL),
 	m_SynergyState(synergyDisconnected),
 	m_ServerConfig(&m_Settings, 5, 3),
 	m_pTempConfigFile(NULL),
 	m_pTrayIcon(NULL),
 	m_pTrayIconMenu(NULL),
-	m_alreadyHidden(false)
+	m_alreadyHidden(false),
+	m_SetupWizard(NULL),
+	m_ElevateProcess(false),
+	m_SuppressElevateWarning(false)
 {
 	setupUi(this);
 
-	createTrayIcon();
 	createMenuBar();
 	loadSettings();
 	initConnections();
+
+	m_pUpdateIcon->hide();
+	m_pUpdateLabel->hide();
+	m_versionChecker.setApp(appPath(appConfig.synergycName()));
+	m_pLabelScreenName->setText(getScreenName());
+	m_pLabelIpAddresses->setText(getIPAddresses());
+
+	m_SetupWizard = new SetupWizard(*this, false);
+
+#if defined(Q_OS_WIN)
+	// ipc must always be enabled, so that we can disable command when switching to desktop mode.
+	connect(&m_IpcClient, SIGNAL(readLogLine(const QString&)), this, SLOT(appendLogRaw(const QString&)));
+	connect(&m_IpcClient, SIGNAL(errorMessage(const QString&)), this, SLOT(appendLogError(const QString&)));
+	connect(&m_IpcClient, SIGNAL(infoMessage(const QString&)), this, SLOT(appendLogNote(const QString&)));
+	m_IpcClient.connectToHost();
+#else
+	// elevate checkbox is only useful on ms windows.
+	m_pElevateCheckBox->hide();
+#endif
+
+    // change default size based on os
+#if defined(Q_OS_MAC)
+	resize(720, 550);
+	setMinimumSize(size());
+#elif defined(Q_OS_LINUX)
+	resize(700, 530);
+	setMinimumSize(size());
+#endif
 }
 
 MainWindow::~MainWindow()
 {
-	stopSynergy();
+	if (appConfig().processMode() == Desktop)
+	{
+		stopDesktop();
+	}
+
 	saveSettings();
+	delete m_SetupWizard;
 }
 
-void MainWindow::start()
+void MainWindow::start(bool firstRun)
 {
-	if (appConfig().autoConnect())
-		startSynergy();
+	onModeChanged(!firstRun && appConfig().autoConnect(), false);
+
+	createTrayIcon();
 
 	// always show. auto-hide only happens when we have a connection.
-	show();
+	showNormal();
 
-	if (appConfig().autoStartPrompt())
+	m_versionChecker.checkLatest();
+}
+
+void MainWindow::onModeChanged(bool startDesktop, bool applyService)
+{
+	refreshApplyButton();
+
+	if (appConfig().processMode() == Service)
 	{
-		int result = QMessageBox::question(this, "Synergy",
-			QObject::tr("Always start Synergy after logging in?"),
-			QMessageBox::Yes | QMessageBox::No);
+		// form is always enabled in service mode.
+		setFormEnabled(true);
 
-		appConfig().setAutoStartPrompt(false);
+		// ensure that the apply button actually does something, since desktop
+		// mode screws around with connecting/disconnecting the action.
+		disconnect(m_pButtonToggleStart, SIGNAL(clicked()), m_pActionStartSynergy, SLOT(trigger()));
+		connect(m_pButtonToggleStart, SIGNAL(clicked()), m_pActionStartSynergy, SLOT(trigger()));
 
-		if (result == QMessageBox::Yes)
+		if (applyService)
 		{
-			appConfig().setAutoStart(true);
-			appConfig().setAutoConnect(true);
+			stopDesktop();
+			startSynergy();
 		}
-
-		appConfig().saveSettings();
 	}
+	else if ((appConfig().processMode() == Desktop) && startDesktop)
+	{
+		stopService();
+		startSynergy();
+	}
+
+	m_pElevateCheckBox->setEnabled(appConfig().processMode() == Service);
+}
+
+void MainWindow::refreshApplyButton()
+{
+	m_pButtonApply->setEnabled(appConfig().processMode() == Service);
 }
 
 void MainWindow::setStatus(const QString &status)
@@ -125,6 +182,9 @@ void MainWindow::createTrayIcon()
 	m_pTrayIcon = new QSystemTrayIcon(this);
 	m_pTrayIcon->setContextMenu(m_pTrayIconMenu);
 
+	connect(m_pTrayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
+			this, SLOT(trayActivated(QSystemTrayIcon::ActivationReason)));
+
 	setIcon(synergyDisconnected);
 
 	m_pTrayIcon->show();
@@ -135,13 +195,11 @@ void MainWindow::createMenuBar()
 	QMenuBar* menubar = new QMenuBar(this);
 	QMenu* pMenuFile = new QMenu(tr("&File"), menubar);
 	QMenu* pMenuEdit = new QMenu(tr("&Edit"), menubar);
-	QMenu* pMenuView = new QMenu(tr("&View"), menubar);
 	QMenu* pMenuWindow = new QMenu(tr("&Window"), menubar);
 	QMenu* pMenuHelp = new QMenu(tr("&Help"), menubar);
 
 	menubar->addAction(pMenuFile->menuAction());
 	menubar->addAction(pMenuEdit->menuAction());
-	menubar->addAction(pMenuView->menuAction());
 #if !defined(Q_OS_MAC)
 	menubar->addAction(pMenuWindow->menuAction());
 #endif
@@ -150,13 +208,11 @@ void MainWindow::createMenuBar()
 	pMenuFile->addAction(m_pActionStartSynergy);
 	pMenuFile->addAction(m_pActionStopSynergy);
 	pMenuFile->addSeparator();
+	pMenuFile->addAction(m_pActionWizard);
 	pMenuFile->addAction(m_pActionSave);
 	pMenuFile->addSeparator();
 	pMenuFile->addAction(m_pActionQuit);
 	pMenuEdit->addAction(m_pActionSettings);
-#if defined(Q_OS_WIN)
-	pMenuEdit->addAction(m_pActionServices);
-#endif
 	pMenuWindow->addAction(m_pActionMinimize);
 	pMenuWindow->addAction(m_pActionRestore);
 	pMenuHelp->addAction(m_pActionAbout);
@@ -168,13 +224,17 @@ void MainWindow::loadSettings()
 {
 	// the next two must come BEFORE loading groupServerChecked and groupClientChecked or
 	// disabling and/or enabling the right widgets won't automatically work
-	m_pRadioExternalConfig->setChecked(settings().value("externalConfig", false).toBool());
-	m_pRadioInternalConfig->setChecked(settings().value("internalConfig", true).toBool());
+        m_pRadioExternalConfig->setChecked(settings().value("useExternalConfig", false).toBool());
+        m_pRadioInternalConfig->setChecked(settings().value("useInternalConfig", true).toBool());
 
 	m_pGroupServer->setChecked(settings().value("groupServerChecked", false).toBool());
 	m_pLineEditConfigFile->setText(settings().value("configFile", QDir::homePath() + "/" + synergyConfigName).toString());
 	m_pGroupClient->setChecked(settings().value("groupClientChecked", true).toBool());
 	m_pLineEditHostname->setText(settings().value("serverHostname").toString());
+
+	m_SuppressElevateWarning = true;
+	m_pElevateCheckBox->setChecked(settings().value("elevateChecked", false).toBool());
+	m_SuppressElevateWarning = false;
 }
 
 void MainWindow::initConnections()
@@ -184,18 +244,16 @@ void MainWindow::initConnections()
 	connect(m_pActionStartSynergy, SIGNAL(triggered()), this, SLOT(startSynergy()));
 	connect(m_pActionStopSynergy, SIGNAL(triggered()), this, SLOT(stopSynergy()));
 	connect(m_pActionQuit, SIGNAL(triggered()), qApp, SLOT(quit()));
-
-	if (m_pTrayIcon)
-		connect(m_pTrayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));
+	connect(&m_versionChecker, SIGNAL(updateFound(const QString&)), this, SLOT(updateFound(const QString&)));
 }
 
 void MainWindow::saveSettings()
 {
 	// program settings
 	settings().setValue("groupServerChecked", m_pGroupServer->isChecked());
-	settings().setValue("externalConfig", m_pRadioExternalConfig->isChecked());
+        settings().setValue("useExternalConfig", m_pRadioExternalConfig->isChecked());
 	settings().setValue("configFile", m_pLineEditConfigFile->text());
-	settings().setValue("internalConfig", m_pRadioInternalConfig->isChecked());
+        settings().setValue("useInternalConfig", m_pRadioInternalConfig->isChecked());
 	settings().setValue("groupClientChecked", m_pGroupClient->isChecked());
 	settings().setValue("serverHostname", m_pLineEditHostname->text());
 
@@ -211,7 +269,7 @@ void MainWindow::setIcon(qSynergyState state)
 		m_pTrayIcon->setIcon(icon);
 }
 
-void MainWindow::iconActivated(QSystemTrayIcon::ActivationReason reason)
+void MainWindow::trayActivated(QSystemTrayIcon::ActivationReason reason)
 {
 	if (reason == QSystemTrayIcon::DoubleClick)
 	{
@@ -236,22 +294,7 @@ void MainWindow::logOutput()
 		{
 			if (!line.isEmpty())
 			{
-				appendLog(line);
-				if (line.contains("has connected") ||
-					line.contains("connected to server"))
-				{
-					// only set connected state and hide, if we get
-					// "has connected" message. this is a little bit
-					// hacky, but it works for now (until we have IPC).
-					setSynergyState(synergyConnected);
-
-					// only hide once after each new connection.
-					if (!m_alreadyHidden && appConfig().autoHide())
-					{
-						hide();
-						m_alreadyHidden = true;
-					}
-				}
+				appendLogRaw(line);				
 			}
 		}
 	}
@@ -261,16 +304,54 @@ void MainWindow::logError()
 {
 	if (m_pSynergy)
 	{
-		QString text(m_pSynergy->readAllStandardError());
-		foreach(QString line, text.split(QRegExp("\r|\n|\r\n")))
-			if (!line.isEmpty())
-				appendLog(line);
+		appendLogRaw(m_pSynergy->readAllStandardError());
 	}
 }
 
-void MainWindow::appendLog(const QString& text)
+void MainWindow::updateFound(const QString &version)
 {
-    m_pLogOutput->append(text);
+	m_pUpdateIcon->show();
+	m_pUpdateLabel->show();
+	m_pUpdateLabel->setText(
+		tr("<p>Version %1 is now available, <a href=\"%2\">visit website</a>.</p>")
+		.arg(version).arg("http://synergy-foss.org"));
+}
+
+void MainWindow::appendLogNote(const QString& text)
+{
+	appendLogRaw("NOTE: " + text);
+}
+
+void MainWindow::appendLogError(const QString& text)
+{
+	appendLogRaw("ERROR: " + text);
+}
+
+void MainWindow::appendLogRaw(const QString& text)
+{
+	foreach(QString line, text.split(QRegExp("\r|\n|\r\n"))) {
+		if (!line.isEmpty()) {
+			m_pLogOutput->append(line);
+			updateStateFromLogLine(line);
+		}
+	}
+}
+
+void MainWindow::updateStateFromLogLine(const QString &line)
+{
+	// TODO: implement ipc connection state messages to replace this hack.
+	if (line.contains("started server") ||
+		line.contains("connected to server"))
+	{
+		setSynergyState(synergyConnected);
+
+		// only hide once after each new connection.
+		if (!m_alreadyHidden && appConfig().autoHide())
+		{
+			hide();
+			m_alreadyHidden = true;
+		}
+	}
 }
 
 void MainWindow::clearLog()
@@ -280,7 +361,14 @@ void MainWindow::clearLog()
 
 void MainWindow::startSynergy()
 {
-	stopSynergy();
+	// TODO: refactor this out into 2 methods.
+	bool desktopMode = appConfig().processMode() == Desktop;
+	bool serviceMode = appConfig().processMode() == Service;
+
+	if (desktopMode)
+	{
+		stopSynergy();
+	}
 
 	setSynergyState(synergyConnecting);
 
@@ -292,42 +380,80 @@ void MainWindow::startSynergy()
 	if (!appConfig().screenName().isEmpty())
 		args << "--name" << appConfig().screenName();
 
-	if (appConfig().gameDevice())
-	{
-		args << "--game-device";
-	}
+		if (appConfig().cryptoMode() != Disabled)
+		{
+			args << "--crypto-mode" << appConfig().cryptoModeString();
+			args << "--crypto-pass" << appConfig().cryptoPass();
+		}
 
-	setSynergyProcess(new QProcess(this));
+	if (desktopMode)
+	{
+		setSynergyProcess(new QProcess(this));
+	}
+	else
+	{
+		// tell client/server to talk to daemon through ipc.
+		args << "--ipc";
+
+#if defined(Q_OS_WIN)
+		// tell the client/server to shut down when a ms windows desk
+		// is switched; this is because we may need to elevate or not
+		// based on which desk the user is in (login always needs
+		// elevation, where as default desk does not).
+		args << "--stop-on-desk-switch";
+#endif
+	}
 
 	if ((synergyType() == synergyClient && !clientArgs(args, app))
 		|| (synergyType() == synergyServer && !serverArgs(args, app)))
 	{
-		stopSynergy();
-		return;
+		if (desktopMode)
+		{
+			stopSynergy();
+			return;
+		}
+	}
+
+	if (desktopMode)
+	{
+		connect(synergyProcess(), SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(synergyFinished(int, QProcess::ExitStatus)));
+		connect(synergyProcess(), SIGNAL(readyReadStandardOutput()), this, SLOT(logOutput()));
+		connect(synergyProcess(), SIGNAL(readyReadStandardError()), this, SLOT(logError()));
 	}
-	
-	connect(synergyProcess(), SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(synergyFinished(int, QProcess::ExitStatus)));
-	connect(synergyProcess(), SIGNAL(readyReadStandardOutput()), this, SLOT(logOutput()));
-	connect(synergyProcess(), SIGNAL(readyReadStandardError()), this, SLOT(logError()));
 
 	// put a space between last log output and new instance.
 	if (!m_pLogOutput->toPlainText().isEmpty())
-		appendLog("");
+		appendLogRaw("");
 
-	appendLog("starting " + QString(synergyType() == synergyServer ? "server" : "client"));
-	appendLog("config file: " + configFilename());
-	appendLog("log level: " + appConfig().logLevelText());
+	appendLogNote("starting " + QString(synergyType() == synergyServer ? "server" : "client"));
+
+	// show command if debug log level...
+	if (appConfig().logLevel() >= 4) {
+		appendLogNote(QString("command: %1 %2").arg(app, args.join(" ")));
+	}
+
+	appendLogNote("config file: " + configFilename());
+	appendLogNote("log level: " + appConfig().logLevelText());
 
 	if (appConfig().logToFile())
-		appendLog("log file: " + appConfig().logFilename());
+		appendLogNote("log file: " + appConfig().logFilename());
 
-	synergyProcess()->start(app, args);
-	if (!synergyProcess()->waitForStarted())
+	if (desktopMode)
 	{
-		stopSynergy();
-		show();
-		QMessageBox::warning(this, tr("Program can not be started"), QString(tr("The executable<br><br>%1<br><br>could not be successfully started, although it does exist. Please check if you have sufficient permissions to run this program.").arg(app)));
-		return;
+		synergyProcess()->start(app, args);
+		if (!synergyProcess()->waitForStarted())
+		{
+			stopSynergy();
+			show();
+			QMessageBox::warning(this, tr("Program can not be started"), QString(tr("The executable<br><br>%1<br><br>could not be successfully started, although it does exist. Please check if you have sufficient permissions to run this program.").arg(app)));
+			return;
+		}
+	}
+
+	if (serviceMode)
+	{
+		QString command(app + " " + args.join(" "));
+		m_IpcClient.sendCommand(command, m_ElevateProcess);
 	}
 }
 
@@ -431,17 +557,16 @@ bool MainWindow::serverArgs(QStringList& args, QString& app)
 
 void MainWindow::stopSynergy()
 {
-	if (synergyProcess())
+	if (appConfig().processMode() == Service)
 	{
-		appendLog("stopping synergy");
-
-		if (synergyProcess()->isOpen())
-			synergyProcess()->close();
-		delete synergyProcess();
-		setSynergyProcess(NULL);
-
-		setSynergyState(synergyDisconnected);
+		stopService();
 	}
+	else if (appConfig().processMode() == Desktop)
+	{
+		stopDesktop();
+	}
+
+	setSynergyState(synergyDisconnected);
 
 	// HACK: deleting the object deletes the physical file, which is
 	// bad, since it could be in use by the Windows service!
@@ -452,6 +577,27 @@ void MainWindow::stopSynergy()
 	m_alreadyHidden = false;
 }
 
+void MainWindow::stopService()
+{
+	// send empty command to stop service from laucning anything.
+	m_IpcClient.sendCommand("", m_ElevateProcess);
+}
+
+void MainWindow::stopDesktop()
+{
+	if (!synergyProcess()) {
+		return;
+	}
+
+	appendLogNote("stopping synergy desktop process");
+
+	if (synergyProcess()->isOpen())
+		synergyProcess()->close();
+
+	delete synergyProcess();
+	setSynergyProcess(NULL);
+}
+
 void MainWindow::synergyFinished(int exitCode, QProcess::ExitStatus)
 {
 	// on Windows, we always seem to have an exit code != 0.
@@ -466,9 +612,6 @@ void MainWindow::synergyFinished(int exitCode, QProcess::ExitStatus)
 #endif
 
 	setSynergyState(synergyDisconnected);
-
-	// do not call stopSynergy() in case of clean synergy shutdown, because this must have (well, should have...)
-	// come from our own delete synergyProcess() in stopSynergy(), so we would do a double-delete...
 }
 
 void MainWindow::setSynergyState(qSynergyState state)
@@ -489,16 +632,24 @@ void MainWindow::setSynergyState(qSynergyState state)
 		m_pButtonToggleStart->setText(tr("&Start"));
 	}
 
-	m_pGroupClient->setEnabled(state == synergyDisconnected);
-	m_pGroupServer->setEnabled(state == synergyDisconnected);
-	m_pActionStartSynergy->setEnabled(state == synergyDisconnected);
-	m_pActionStopSynergy->setEnabled(state == synergyConnected);
+	bool connected = state == synergyConnected;
+
+	// only disable controls in desktop mode. in service mode, we can use the apply button.
+	if (appConfig().processMode() == Desktop)
+	{
+		setFormEnabled(!connected);
+	}
+
+	m_pActionStartSynergy->setEnabled(!connected);
+	m_pActionStopSynergy->setEnabled(connected);
 
 	switch (state)
 	{
-	case synergyConnected:
-		setStatus(tr("Synergy is running."));
+	case synergyConnected: {
+		QString mode(appConfig().processMode() == Service ? tr("service mode") : tr("desktop mode"));
+		setStatus(tr("Synergy is running (%1).").arg(mode));
 		break;
+	}
 	case synergyConnecting:
 		setStatus(tr("Synergy is starting."));
 		break;
@@ -512,6 +663,12 @@ void MainWindow::setSynergyState(qSynergyState state)
 	m_SynergyState = state;
 }
 
+void MainWindow::setFormEnabled(bool enabled)
+{
+	m_pGroupClient->setEnabled(enabled);
+	m_pGroupServer->setEnabled(enabled);
+}
+
 void MainWindow::setVisible(bool visible)
 {
 	m_pActionMinimize->setEnabled(visible);
@@ -529,6 +686,50 @@ void MainWindow::setVisible(bool visible)
 #endif
 }
 
+QString MainWindow::getIPAddresses()
+{
+	QList<QHostAddress> addresses = QNetworkInterface::allAddresses();
+
+	bool hinted = false;
+	QString result;
+	for (int i = 0; i < addresses.size(); i++) {
+		if (addresses[i].protocol() == QAbstractSocket::IPv4Protocol &&
+			addresses[i] != QHostAddress(QHostAddress::LocalHost)) {
+
+			QString address = addresses[i].toString();
+			QString format = "%1, ";
+
+			// usually 192.168.x.x is a useful ip for the user, so indicate
+			// this by making it bold.
+			if (!hinted && address.startsWith("192.168")) {
+				hinted = true;
+				format = "<b>%1</b>, ";
+			}
+
+			result += format.arg(address);
+		}
+	}
+
+	if (result == "") {
+		return tr("Unknown");
+	}
+
+	// remove trailing comma.
+	result.chop(2);
+
+	return result;
+}
+
+QString MainWindow::getScreenName()
+{
+	if (appConfig().screenName() == "") {
+		return QHostInfo::localHostName();
+	}
+	else {
+		return appConfig().screenName();
+	}
+}
+
 bool MainWindow::on_m_pButtonBrowseConfigFile_clicked()
 {
 	QString fileName = QFileDialog::getOpenFileName(this, tr("Browse for a synergys config file"), QString(), synergyConfigFilter);
@@ -563,14 +764,15 @@ void MainWindow::on_m_pActionAbout_triggered()
 
 void MainWindow::on_m_pActionSettings_triggered()
 {
+	ProcessMode lastProcessMode = appConfig().processMode();
+
 	SettingsDialog dlg(this, appConfig());
 	dlg.exec();
-}
 
-void MainWindow::on_m_pActionServices_triggered()
-{
-	WindowsServices dlg(this, appConfig());
-	dlg.exec();
+	if (lastProcessMode != appConfig().processMode())
+	{
+		onModeChanged(true, true);
+	}
 }
 
 void MainWindow::on_m_pButtonConfigureServer_clicked()
@@ -578,3 +780,35 @@ void MainWindow::on_m_pButtonConfigureServer_clicked()
 	ServerConfigDialog dlg(this, serverConfig(), appConfig().screenName());
 	dlg.exec();
 }
+
+void MainWindow::on_m_pActionWizard_triggered()
+{
+	m_SetupWizard->show();
+}
+
+void MainWindow::on_m_pElevateCheckBox_toggled(bool checked)
+{
+	if (checked && !m_SuppressElevateWarning) {
+		int r = QMessageBox::warning(
+			this, tr("Elevate Synergy"),
+			tr("Are you sure you want to elevate Synergy?\n\n"
+			   "This allows Synergy to interact with elevated processes "
+			   "and the UAC dialog, but can cause problems with non-elevated "
+			   "processes. Elevate Synergy only if you really need to."),
+			QMessageBox::Yes | QMessageBox::No);
+
+		if (r != QMessageBox::Yes) {
+			m_pElevateCheckBox->setChecked(false);
+			return;
+		}
+	}
+
+	m_ElevateProcess = checked;
+	settings().setValue("elevateChecked", checked);
+	settings().sync();
+}
+
+void MainWindow::on_m_pButtonApply_clicked()
+{
+	startSynergy();
+}
